/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.db;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.RandomAccessFile;
import java.util.Vector;

import org.apache.commons.io.FileUtils;

import re.jpayet.mentdb.core.command.Command;
import re.jpayet.mentdb.core.command.MQLValue;
import re.jpayet.mentdb.core.db.file.data.DataFile;
import re.jpayet.mentdb.core.db.file.data.DataFilePool;
import re.jpayet.mentdb.core.db.file.index.IndexFile;
import re.jpayet.mentdb.core.db.file.index.IndexFilePool;
import re.jpayet.mentdb.core.db.file.transaction.Transaction;
import re.jpayet.mentdb.core.entity.concentration.ConcentrationManager;
import re.jpayet.mentdb.core.entity.word.WordManager;
import re.jpayet.mentdb.core.job.JobManager;
import re.jpayet.mentdb.core.parameter.ParameterManager;
import re.jpayet.mentdb.core.script.ScriptManager;
import re.jpayet.mentdb.core.sequence.SequenceManager;
import re.jpayet.mentdb.core.session.SessionThread;
import re.jpayet.mentdb.core.user.GroupManager;
import re.jpayet.mentdb.server.Start;
import re.jpayet.mentdb.tools.Misc;

//Manage the database
public class Database {
	
	public static boolean isNotInit = true;
	
	//Initialization
	public static void init(String user) throws Exception {

		//######################################################################
		//#INITIALIZATION#######################################################
		//######################################################################
		
		//Initialize the global parameter
		ParameterManager.init(user);
		
		//Initialize the sequence manager
		SequenceManager.init(user);
		
		//Initialize the concentration depth word
		ConcentrationManager.init(user);
		
		//Initialize the job manager
		JobManager.init(user);
		
		//Initialize the group manager
		GroupManager.init(user);
		
		//Initialize the script manager
		ScriptManager.init(user);

		//######################################################################
		//#BASIC INTEGRATION####################################################
		//######################################################################
		
		isNotInit = false;
		
		FileInputStream fis = new FileInputStream(new File("docs"+File.separator+"mql"+File.separator+"basic-integration.mql"));
		 
		//Construct BufferedReader from InputStreamReader
		BufferedReader br = new BufferedReader(new InputStreamReader(fis));
		
		SessionThread mentdbInternalSession = new SessionThread(null);
		mentdbInternalSession.user = "mentdb";
	 
		String line = null, mql = "";
		while ((line = br.readLine()) != null) {
			
			if (Misc.lrtrim(line).equals("#VALID MQL BLOCK;")) {

				Vector<Vector<MQLValue>> inputVectorCmds = Misc.splitCommand(mql);
				
				for(int i=0;i<inputVectorCmds.size();i++) {
				
					try {
						
						Vector<MQLValue> inputVectorCmd = inputVectorCmds.get(i);
						Command.execute(mentdbInternalSession, inputVectorCmd, mentdbInternalSession.env);
						Transaction.commit("mentdb");
						
					} catch (Exception e) {
						
						br.close();
						
						throw new Exception(e.getMessage()+" / "+Misc.vectorToStringMsg(inputVectorCmds.get(i))); 
						
					}

				}
				
				mql = "";
				
			} else {
				
				mql += line+"\n";
				
			}	
		}
		
		br.close();
		
		isNotInit = true;

		//######################################################################
		//#BASIC EMOTIONS#######################################################
		//######################################################################
		/*
		//Create the joie emotion
		String thought_joie = EmotionManager.create(user, "joie", "fr", 0, "positive", false, null, 0);
		EmotionManager.create(user, "sérénité", "fr", 0, "positive", false, thought_joie, 0);
		EmotionManager.create(user, "extase", "fr", 2, "positive", true, thought_joie, 0);
		
		//Create the confiance emotion
		String thought_confiance = EmotionManager.create(user, "confiance", "fr", 0, "positive", false, null, 0);
		EmotionManager.create(user, "acceptation", "fr", 2, "positive", false, thought_confiance, 0);
		EmotionManager.create(user, "admiration", "fr", 2, "positive", true, thought_confiance, 0);
		
		//Create the peur emotion
		String thought_peur = EmotionManager.create(user, "peur", "fr", 0, "negative", false, null, 0);
		EmotionManager.create(user, "appréhension", "fr", 2, "negative", false, thought_peur, 0);
		EmotionManager.create(user, "terreur", "fr", 0, "negative", true, thought_peur, 0);
		
		//Create the surprise emotion
		String thought_surprise = EmotionManager.create(user, "surprise", "fr", 0, "neutral", false, null, 0);
		EmotionManager.create(user, "distration", "fr", 0, "neutral", false, thought_surprise, 0);
		EmotionManager.create(user, "étonnement", "fr", 2, "neutral", true, thought_surprise, 0);
		
		//Create the tristesse emotion
		String thought_tristesse = EmotionManager.create(user, "tristesse", "fr", 0, "negative", false, null, 0);
		EmotionManager.create(user, "songerie", "fr", 0, "negative", false, thought_tristesse, 0);
		EmotionManager.create(user, "chagrin", "fr", 1, "negative", true, thought_tristesse, 0);
		
		//Create the dégoût emotion
		String thought_dégoût = EmotionManager.create(user, "dégoût", "fr", 1, "negative", false, null, 0);
		EmotionManager.create(user, "ennui", "fr", 2, "negative", false, thought_dégoût, 1);
		EmotionManager.create(user, "aversion", "fr", 2, "negative", true, thought_dégoût, 1);
		
		//Create the colère emotion
		String thought_colère = EmotionManager.create(user, "colère", "fr", 0, "negative", false, null, 0);
		EmotionManager.create(user, "contrariété", "fr", 0, "negative", false, thought_colère, 0);
		EmotionManager.create(user, "rage", "fr", 0, "negative", true, thought_colère, 0);
		
		//Create the anticipation emotion
		String thought_anticipation = EmotionManager.create(user, "anticipation", "fr", 2, "neutral", false, null, 0);
		EmotionManager.create(user, "intérêt", "fr", 2, "neutral", false, thought_anticipation, 0);
		EmotionManager.create(user, "vigilance", "fr", 0, "neutral", true, thought_anticipation, 0);
		*/
		//Commit the transaction
		Transaction.commit(user);
		
	}
	
	//Import all french words
	public static void importFrenchWords(String user) throws Exception {
		
		//Initialization
	    String line="";
	    String words = "";
		
		try (BufferedReader br = new BufferedReader(
		         new InputStreamReader(
		         new FileInputStream("tools"+File.separator+"fr_test_words.csv"), "UTF-8"))) {
		    int i = 0;
		    int maxload = 142695;
		    @SuppressWarnings("unused")
			int max = 142695;
		    
		    System.out.print("\rImport French Words > "+"  "+((int)(1)/10)+"/100%");
		    
		    while ((line = br.readLine()) != null) {
		       // process the line.
		    	
		    	if (i==maxload) break;
		    	
		    	if (i>0) {
		    		words = line.split("\t", -1)[0];
		    		int nbWord = Misc.size(words, " ");
		    		for(int j=1;j<=nbWord;j++) {
		    			
		    			String tmpStr = "";
						int percent = ((int)(i+1)*100/maxload);
						if (percent<10) tmpStr+=" ";
						if (percent<100) tmpStr+=" ";
							
						System.out.print("\rImport French Words > "+tmpStr+percent+"/100%");
						
		    			WordManager.create(user, Misc.atom(words, j, " "), "", "fr");
		    			
		    		}
		    	}
		    	
		    	i++;
		    }
		    
		    System.out.print("\rImport French Words > 100/100%");
		    
		} catch (Exception e) {
			throw new Exception(words);
		}
				
	}
	
	//Recreate the first index file
	
	
	//Open the database
	public void open() throws IOException {
		
		//Create the data folder if does not exist
		if (!(new File("data").exists())) {
			
			(new File("data")).mkdir();
			
		}
		
		//Create the data/index folder if does not exist
		if (!(new File("data"+File.separator+"index").exists())) {
			
			(new File("data"+File.separator+"index")).mkdir();
			
			//Create the data/data/0 folder if does not exist
			if (!(new File("data"+File.separator+"index"+File.separator+"0").exists())) {
				
				(new File("data"+File.separator+"index"+File.separator+"0")).mkdir();
				
			}
			
			String tmpStr = "";
			if (((int)(1)/10)<10) tmpStr+=" ";
			if (((int)(1)/10)<100) tmpStr+=" ";
				
			System.out.print("\rCreate the database > "+tmpStr+((int)(1)/10)+"/100%");
			
			//Create the first index file
			(new File("tools"+File.separator+"brain0.idx")).delete();
			IndexFile indexFile = new IndexFile(0, "tools"+File.separator+"brain0.idx");
			indexFile.open();
			indexFile.indexFile.writeLong(0);
			indexFile.addExtraEntries(16);
			indexFile.close();

			FileUtils.copyFile(new File("tools"+File.separator+"brain0.idx"), new File("data"+File.separator+"index"+File.separator+"0"+File.separator+"brain0.idx"));
			
			for(int i=1;i<Start.NB_INDEX_FILE;i++) {

				tmpStr = "";
				int percent = ((int)(i+1)*100/Start.NB_INDEX_FILE);
				if (percent<10) tmpStr+=" ";
				if (percent<100) tmpStr+=" ";
					
				System.out.print("\rCreate the database > "+tmpStr+percent+"/100%");
				
				//Create the data/data/id folder if does not exist
				if (!(new File("data"+File.separator+"index"+File.separator+i).exists())) {
					
					(new File("data"+File.separator+"index"+File.separator+i)).mkdir();
					
				}
				
				//Copy the first file
				FileUtils.copyFile(new File("data"+File.separator+"index"+File.separator+"0"+File.separator+"brain0.idx"), new File("data"+File.separator+"index"+File.separator+i+File.separator+"brain"+i+".idx"));
				
			}
			
		}
		
		//Create the data/data folder if does not exist
		if (!(new File("data"+File.separator+"data").exists())) {
			
			(new File("data"+File.separator+"data")).mkdir();
			
			DataFile dataFile = new DataFile(0, "data"+File.separator+"data"+File.separator+"0"+File.separator+"brain0.dat");
			dataFile.open();
			dataFile.close();
			
		}
		
		//Create the data/transaction folder if does not exist
		if (!(new File("data"+File.separator+"transaction").exists())) {
			
			(new File("data"+File.separator+"transaction")).mkdir();
			
		}
		
		//Create the data/transaction folder if does not exist
		if (!(new File("data"+File.separator+"transaction"+File.separator+"mentdb").exists())) {
			
			(new File("data"+File.separator+"transaction"+File.separator+"mentdb")).mkdir();
			
		}

		//Open the log file
		Transaction.logFiles.put("mentdb", new RandomAccessFile("data"+File.separator+"transaction"+File.separator+"mentdb"+File.separator+"rollback.log", "rw"));
		
	}
	
	//Close the database
	public void close() throws IOException {
		
		//Close all data files
		DataFilePool.close();

		//Close all index files
		IndexFilePool.close();
		
	}

}
