/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.entity.thought;

import java.io.IOException;
import java.util.Set;
import java.util.Vector;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

import re.jpayet.mentdb.core.db.Record;
import re.jpayet.mentdb.core.db.file.experience.ExperienceManagerL1;
import re.jpayet.mentdb.core.db.file.json.JsonManager;
import re.jpayet.mentdb.core.entity.concentration.ConcentrationManager;
import re.jpayet.mentdb.core.entity.language.LanguageManager;
import re.jpayet.mentdb.core.entity.relation.RelationManager;
import re.jpayet.mentdb.core.entity.word.WordManager;
import re.jpayet.mentdb.core.fx.StringFx;
import re.jpayet.mentdb.core.sequence.SequenceManager;
import re.jpayet.mentdb.tools.Misc;

public class ThoughtManager {
	
	@SuppressWarnings("unchecked")
	public static JSONObject pearlNecklaceRelations(String user, String key) throws Exception {
		
		//Initialization
		int[] autoincrement = {0};
		JSONObject result = new JSONObject();
		JSONObject nodes = new JSONObject();result.put("n", nodes);
		JSONArray edges = new JSONArray();result.put("e", edges);
		String lastKey = "";
		
		int concentration = ConcentrationManager.getConcentrationDepth("C[graph-relation]");
		
		JSONObject rec = Record.getNode(key);
		
		if (rec==null) {

			throw new Exception("Sorry, the thought/relation id "+key+" does not exist.");

		}
		
		if (key.startsWith("TH[")) {
			
			String currentThouhght = "Y"+autoincrement[0]+"_"+key;
			nodes.put(currentThouhght, key);
			lastKey = currentThouhght;
			
		} else {
			
			String currentRelation = "V"+autoincrement[0]+"_"+key;
			nodes.put(currentRelation, key);
			lastKey = currentRelation;

			ThoughtManager.findParent(user, key, nodes, edges, autoincrement, lastKey);
			ThoughtManager.findCircle(user, key, nodes, edges, autoincrement, lastKey);
			
			JSONArray thoughts = RelationManager.showThoughtsRecursively(user, key);
			String thoughtLastKey = currentRelation;
			for(int i=0;i<thoughts.size();i++) {
				
				String thoughtId = thoughts.get(i)+"";
				
				autoincrement[0]++;
				String currentThouhght = "T"+autoincrement[0]+"_"+thoughtId;
				
				nodes.put(currentThouhght, thoughtId);
				JSONObject e = new JSONObject();
				e.put("k", "$"+thoughtLastKey+"_"+currentThouhght);
				e.put("n1", thoughtLastKey);
				e.put("n2", currentThouhght);
				edges.add(e);
				
				thoughtLastKey = currentThouhght;
				
			}
			
		}
		
		branch(user, key, "G", nodes, edges, autoincrement, lastKey, concentration);
		branch(user, key, "L", nodes, edges, autoincrement, lastKey, concentration);
		branch(user, key, "I", nodes, edges, autoincrement, lastKey, concentration);
		branch(user, key, "O", nodes, edges, autoincrement, lastKey, concentration);
		branch(user, key, "U", nodes, edges, autoincrement, lastKey, concentration);
		branch(user, key, "S", nodes, edges, autoincrement, lastKey, concentration);
		branch(user, key, "D", nodes, edges, autoincrement, lastKey, concentration);
		branch(user, key, "A", nodes, edges, autoincrement, lastKey, concentration);
		branch(user, key, "R", nodes, edges, autoincrement, lastKey, concentration);
		
		if (key.startsWith("R")) {
			
			JSONArray tabLinks = RelationManager.listTabLinks(user, key);
			
			for(int i=0;i<tabLinks.size();i++) {
				
				String curTabLink = (String) tabLinks.get(i);
				autoincrement[0]++;
				nodes.put("M"+autoincrement[0]+"_"+curTabLink, (i+1)+"");
				JSONObject e = new JSONObject();
				e.put("k", "$"+lastKey+"_"+autoincrement[0]);
				e.put("n1", lastKey);
				e.put("n2", "M"+autoincrement[0]+"_"+curTabLink);
				edges.add(e);
				lastKey = "M"+autoincrement[0]+"_"+curTabLink;
				
				branch(user, curTabLink, "G", nodes, edges, autoincrement, lastKey, concentration);
				branch(user, curTabLink, "L", nodes, edges, autoincrement, lastKey, concentration);
				branch(user, curTabLink, "I", nodes, edges, autoincrement, lastKey, concentration);
				branch(user, curTabLink, "O", nodes, edges, autoincrement, lastKey, concentration);
				branch(user, curTabLink, "U", nodes, edges, autoincrement, lastKey, concentration);
				branch(user, curTabLink, "S", nodes, edges, autoincrement, lastKey, concentration);
				branch(user, curTabLink, "D", nodes, edges, autoincrement, lastKey, concentration);
				branch(user, curTabLink, "A", nodes, edges, autoincrement, lastKey, concentration);
				branch(user, curTabLink, "R", nodes, edges, autoincrement, lastKey, concentration);
				
			}
			
		}
		
		return result;
		
	}
	

	@SuppressWarnings("unchecked")
	public static void findCircle(String user, String key, JSONObject nodes, JSONArray edges, int[] autoincrement, String lastKey) throws Exception {
		
		JSONObject bd = Record.getNode(key);
		JSONObject currentCircle = null;
		String circle_id = (String) bd.get("c");
		if (circle_id==null) currentCircle =  new JSONObject();
		else {
			
			currentCircle =  Record.getNode(circle_id);
			
		}
		
		JSONObject langNode = new JSONObject();
		JSONObject rNode = new JSONObject();
		
		for(Object o1 : currentCircle.keySet()) {
			
			//get the current key
			String k = (String) o1;
			String l = Misc.atom(k, 1, " ");
			String r1 = Misc.atom(k, 2, " ");
			String r2 = Misc.atom(k, 3, " ");
			
			if (!langNode.containsKey(l)) {
				
				autoincrement[0]++;
				String currentBranch = "C"+autoincrement[0]+"_"+l;
				
				JSONObject e = null;
				nodes.put(currentBranch, l);
				e = new JSONObject();
				e.put("k", lastKey+"_"+currentBranch);
				e.put("n1", lastKey);
				e.put("n2", currentBranch);
				edges.add(e);
				
				langNode.put(l, currentBranch);
				
			}
			
			if (!rNode.containsKey(r2)) {
				
				if (r1.equals(key)) {
					
					autoincrement[0]++;
					String currentRelation = "R"+autoincrement[0]+"_"+r2;
					nodes.put(currentRelation, r2);
					JSONObject e = new JSONObject();
					e.put("k", langNode.get(l)+"_"+currentRelation);
					e.put("n1", langNode.get(l));
					e.put("n2", currentRelation);
					edges.add(e);
					
					rNode.put(r2, null);
					
				}
				
			}
			
		}
		
	}
	

	@SuppressWarnings("unchecked")
	public static void findParent(String user, String key, JSONObject nodes, JSONArray edges, int[] autoincrement, String lastKey) throws Exception {
		
		JSONObject bd = Record.getNode(key);
		String parentId = (String) bd.get("p");
		
		if (parentId!=null && !parentId.equals("")) {
			
			autoincrement[0]++;
			nodes.put("V"+autoincrement[0]+"_"+parentId, parentId);
			JSONObject e = new JSONObject();
			e.put("k", "$"+lastKey+"_"+autoincrement[0]);
			e.put("n1", lastKey);
			e.put("n2", "V"+autoincrement[0]+"_"+parentId);
			edges.add(e);
			
			lastKey = "V"+autoincrement[0]+"_"+parentId;
			
			findParent(user, parentId, nodes, edges, autoincrement, lastKey);
			
		}
		
	}
	
	@SuppressWarnings("unchecked")
	public static void branch(String user, String key, String branch, JSONObject nodes, JSONArray edges, int[] autoincrement, String lastKey, int depth) throws Exception {
		
		JSONArray list = RelationManager.showTabLinks(user, key, branch, depth);
		
		if (list.size()>0) {
			
			autoincrement[0]++;
			String currentBranch = "B"+autoincrement[0]+"_"+branch;
			
			nodes.put(currentBranch, branch);
			JSONObject e = new JSONObject();
			e.put("k", lastKey+"_"+currentBranch);
			e.put("n1", lastKey);
			e.put("n2", currentBranch);
			edges.add(e);
			
			lastKey = currentBranch;
			
		}
		
		for(int i=0;i<list.size();i++) {
			
			String tabLink = ""+list.get(i);
			String rel = Misc.atom(tabLink, 1, " ").substring(1);
			
			autoincrement[0]++;
			String currentRelation = "R"+autoincrement[0]+"_"+rel;
			nodes.put(currentRelation, rel);
			JSONObject e = new JSONObject();
			e.put("k", lastKey+"_"+currentRelation);
			e.put("n1", lastKey);
			e.put("n2", currentRelation);
			edges.add(e);
						
			lastKey = currentRelation;
			
		}
		
	}

	//Add a new thought
	public static String create(String user, String wordName, String separator, String lang) throws Exception {
		
		if (separator==null || separator.equals("")) {
			
			return createThought(user, wordName, lang);
			
		} else {
		
			//Split words
			String[] words = Misc.splitWords(wordName, separator);
			
			return createThoughtComposed(user, wordName, separator, lang, words);
			
		}
		
	}
	
	//Add a new thought
	public static String createThought(String user, String wordName, String lang) throws Exception {
		
		
		if (wordName.length()==0) {

			throw new Exception("Sorry, the word "+wordName+" is not valid [required].");

		}
		
		//Try to get the record
		JSONObject rec = Record.getNode("W["+wordName+"]");

		if (rec==null) {

			throw new Exception("Sorry, the word "+wordName+" does not exist.");

		}
		
		//The default something thought
		String thoughtId = "";
		
		if (wordName.equals("quelque chose") && lang.equals("fr")) {
			
			thoughtId = "TH[]";
			
		} else {
			
			thoughtId = "TH["+SequenceManager.incr(user, "thought")+"]";
			while (Record.getNode(thoughtId)!=null) {
				
				thoughtId = "TH["+SequenceManager.incr(user, "thought")+"]";
				
			}
			
		}
		
		ExperienceManagerL1.addNodeTabLink(user, "W["+wordName+"]", thoughtId, lang, "word", true, true, 0);
		
		return thoughtId;

	}
	
	//Add a new thought (composed)
	@SuppressWarnings("unchecked")
	public static String createThoughtComposed(String user, String wordName, String separator, String lang, String[] words) throws Exception {
		
		//Initialization
		String gWordName = wordName;
		Vector<String> allThoughtIds = new Vector<String>();
		
		if (wordName.length()==0) {

			throw new Exception("Sorry, the word "+wordName+" is not valid [required].");

		}
		
		//Try to get the record
		JSONObject rec = Record.getNode("W["+wordName+"]");

		if (rec==null) {

			throw new Exception("Sorry, the word '"+wordName+"' does not exist.");

		}
		
		for(int iWord = 0;iWord<words.length;iWord++) {
			
			wordName = words[iWord];
			
			//The default something thought
			String thoughtId = "";
			
			thoughtId = "TH["+SequenceManager.incr(user, "thought")+"]";
			while (Record.getNode(thoughtId)!=null) {
				
				thoughtId = "TH["+SequenceManager.incr(user, "thought")+"]";
				
			}
			
			ExperienceManagerL1.addNodeTabLink(user, "W["+wordName+"]", thoughtId, lang, "word", true, true, 0);
			
			JSONObject bd = Record.getNode(thoughtId);
			((JSONObject) ((JSONObject) bd.get("ud")).get(lang+" W["+wordName+"]")).put("cw", gWordName);
			
			Record.update(user, thoughtId, bd.toJSONString());
			
			allThoughtIds.add(thoughtId);
			
		}
		
		//Merge thoughts for all separate words
		for(int j=1;j<allThoughtIds.size();j++) {
			ThoughtManager.merge(user, allThoughtIds.get(0), allThoughtIds.get(j));
		}
		
		String thoughtId = "";
		
		if (gWordName.equals("quelque chose") && lang.equals("fr")) {
			
			thoughtId = "TH[]";
			
		} else {
		
			thoughtId = "TH["+SequenceManager.incr(user, "thought")+"]";
			while (Record.getNode(thoughtId)!=null) {
				
				thoughtId = "TH["+SequenceManager.incr(user, "thought")+"]";
				
			}
			
		}
		
		ExperienceManagerL1.addNodeTabLink(user, "W["+gWordName+"]", thoughtId, lang, "word", true, true, 0);
		
		JSONObject bd = Record.getNode(thoughtId);
		((JSONObject) ((JSONObject) bd.get("ud")).get(lang+" W["+gWordName+"]")).put("cw", gWordName);
		
		Record.update(user, thoughtId, bd.toJSONString());
		
		ThoughtManager.merge(user, thoughtId, allThoughtIds.get(0));
		
		return thoughtId;

	}
	
	//Get all thoughts linked with a word
	public static JSONArray list(String user, String wordName, String lang) throws Exception {

		if (wordName.length()==0) {

			throw new Exception("Sorry, the word "+wordName+" is not valid [required].");

		}
		
		//Try to get the record
		JSONObject rec = Record.getNode("W["+wordName+"]");
		
		if (rec==null) {

			throw new Exception("Sorry, the word "+wordName+" does not exist.");

		}
		
		return ExperienceManagerL1.getAllNodeTabLinks(user, "W["+wordName+"]", lang);

	}
	
	//Get all thoughts linked with a word
	@SuppressWarnings("unchecked")
	public static JSONArray list(String user, String wordName) throws Exception {

		if (wordName.length()==0) {

			throw new Exception("Sorry, the word "+wordName+" is not valid [required].");

		}

		//Try to get the record
		JSONObject rec = Record.getNode("W["+wordName+"]");

		if (rec==null) {

			throw new Exception("Sorry, the word "+wordName+" does not exist.");

		}
		
		//Initialization
		JSONArray result = new JSONArray();
		
		//Get all languages
		JSONArray languages = LanguageManager.showAll();
		for(int i=0;i<languages.size();i++) {
			
			//Get the thought
			String lang = languages.get(i)+"";
			
			try {
			
				JSONArray array = ExperienceManagerL1.getAllNodeTabLinksForStimulation(user, "W["+wordName+"]", lang);
				for(int k=0;k<array.size();k++) {
					result.add(array.get(k));
				}
				
			} catch (Exception e) {}
			
		}
		
		return result;

	}
	
	//Get words from thought
	@SuppressWarnings("unchecked")
	public static JSONArray getWords(String thoughtId) throws IOException {
		
		//Initialization
		JSONArray result = new JSONArray();
		
		if (thoughtId==null) {
			
			return result;
		
		} else {
		
			JSONObject bwTmp = Record.getNode(thoughtId);
			
			if (bwTmp==null) {
				
				return result;
				
			}
			
			Set<String> entrySetTmp = ((JSONObject) bwTmp.get("ud")).keySet();
			for(String langTmp : entrySetTmp) {
				
				JSONObject obj = new JSONObject();
				JSONObject curNode = ((JSONObject) ((JSONObject) bwTmp.get("ud")).get(langTmp));
				if (curNode.containsKey("cw")) obj.put("word", "W["+curNode.get("cw")+"]");
				else obj.put("word", Misc.atom(langTmp, 2, " "));
				obj.put("lang", Misc.atom(langTmp, 1, " "));
				if (!result.contains(obj)) result.add(obj);
				
			}
			
			return result;
		}
		
	}
	
	//Get words from thought
	@SuppressWarnings("unchecked")
	public static JSONArray getWords(String thoughtId, String lang) throws IOException {
		
		//Initialization
		JSONArray result = new JSONArray();
		
		if (thoughtId==null) {
			
			return result;
		
		} else {
		
			JSONObject bwTmp = Record.getNode(thoughtId);
			
			if (bwTmp==null) {
				
				return result;
				
			}
			
			Set<String> entrySetTmp = ((JSONObject) bwTmp.get("ud")).keySet();
			for(String langTmp : entrySetTmp) {
				
				if (Misc.atom(langTmp, 1, " ").equals(lang)) {
					
					JSONObject obj = new JSONObject();
					JSONObject curNode = ((JSONObject) ((JSONObject) bwTmp.get("ud")).get(langTmp));
					if (curNode.containsKey("cw")) obj.put("word", "W["+curNode.get("cw")+"]");
					else obj.put("word", Misc.atom(langTmp, 2, " "));
					obj.put("lang", Misc.atom(langTmp, 1, " "));
					if (!result.contains(obj)) result.add(obj);
					
				}
				
			}
			
			if (result.size()==0) {
				
				for(String langTmp : entrySetTmp) {
					
					JSONObject obj = new JSONObject();
					JSONObject curNode = ((JSONObject) ((JSONObject) bwTmp.get("ud")).get(langTmp));
					if (curNode.containsKey("cw")) obj.put("word", "W["+curNode.get("cw")+"]");
					else obj.put("word", Misc.atom(langTmp, 2, " "));
					obj.put("lang", Misc.atom(langTmp, 1, " "));
					if (!result.contains(obj)) result.add(obj);
					
				}
				
			}
			
			return result;
		}
		
	}

	//Stimulate relations
	@SuppressWarnings("unchecked")
	public static void stimulateRelations(String user, String thoughtId, JSONArray rels) throws Exception {

		if (thoughtId.length()==0) {

			throw new Exception("Sorry, the thought id "+thoughtId+" is not valid [required].");

		}

		//Try to get the thought
		JSONObject recThought = Record.getNode(thoughtId);

		if (recThought==null) {

			throw new Exception("Sorry, the thought "+thoughtId+" does not exist.");

		}
		
		//Get all relations
		JSONArray relations = RelationManager.showTabLinks(user, thoughtId, "L", ConcentrationManager.getConcentrationDepth("C[relation]"));
		
		//Parse all relations
		for(int i = relations.size()-1;i>=0;i--) {
			
			String relationId = Misc.atom((relations.get(i)+""), 1, " ").substring(1);
			
			//Stimulate the relation
			RelationManager.stimulate(user, relationId, true);
			
			JSONArray words = RelationManager.showWords(user, relationId);
			
			for(int j=0;j<words.size();j++) {
				
				JSONArray cur = (JSONArray) words.get(j);
				
				for(int k=0;k<cur.size();k++) {
					
					JSONObject obj = (JSONObject) cur.get(k);

					String lang = obj.get("lang")+"";
					String word = (obj.get("word")+"");
					word = word.substring(2, word.length()-1);
					
					String[] wordList = word.split(" ");

					for(int z=0;z<wordList.length;z++) {
						WordManager.stimulate(user, wordList[z], lang);
					}
					
				}
				
			}
			
			rels.add(0, relationId);
			
		}
		
	}

	//Stimulate a thought
	@SuppressWarnings("unchecked")
	public static void stimulate(String user, String thoughtId) throws Exception {

		if (thoughtId.length()==0) {

			throw new Exception("Sorry, the thought id "+thoughtId+" is not valid [required].");

		}
		
		JSONObject th = Record.getNode(thoughtId);

		if (th==null) {

			throw new Exception("Sorry, the thought "+thoughtId+" does not exist.");

		}

		//Try to get the thought
		JSONObject recThought = ((JSONObject) JsonManager.load(th.toJSONString()));
		
		//Get all links
		Set<String> entrySet = ((JSONObject) recThought.get("ud")).keySet();
		for(String entry : entrySet) {
			
			String lang = entry.substring(0, 2);
			String word = entry.substring(3, entry.length());
			
			//Delete the node
			ExperienceManagerL1.deleteNodeTabLink(user, word, thoughtId, lang);
			
			String firstThought = ExperienceManagerL1.getFirstTabLink(user, word, lang, true);
			JSONObject obj = new JSONObject();
			obj.put("utl", null); //First tab link
			obj.put("dtl", firstThought); //Last tab link
			
			((JSONObject) recThought.get("ud")).put(entry, obj);
			
			recThought.put("w", Integer.parseInt(""+recThought.get("w"))+1);
			
			Record.update(user, thoughtId, recThought.toJSONString());
			
			//Recreate the thought
			ExperienceManagerL1.mergeNodeTabLink(user, word, thoughtId, lang, "word", true, false);
			
		}
		
	}

	//Merge two thought
	public static void merge(String user, String thoughtId1, String thoughtId2) throws Exception {

		if (thoughtId1.length()==0) {

			throw new Exception("Sorry, the thought id "+thoughtId1+" is not valid [required].");

		}

		if (thoughtId2.length()==0) {

			throw new Exception("Sorry, the thought id "+thoughtId2+" is not valid [required].");

		}

		if (thoughtId1.equals(thoughtId2)) {

			throw new Exception("Sorry, it is the same thought.");

		}

		//Try to get the thought1
		JSONObject recThought1 = Record.getNode(thoughtId1);
		
		if (recThought1==null) {

			throw new Exception("Sorry, the thought "+thoughtId1+" does not exist.");

		}
		
		//Try to get the thought2
		JSONObject recThought2 = Record.getNode(thoughtId2);

		if (recThought2==null) {

			throw new Exception("Sorry, the thought "+thoughtId2+" does not exist.");

		}
		
		ExperienceManagerL1.merge(user, thoughtId1, thoughtId2);

	}
	
	//Delete a thought
	public static void delete(String user, String thoughtId) throws Exception {

		if (thoughtId.length()==0) {

			throw new Exception("Sorry, the thought id "+thoughtId+" is not valid [required].");

		}

		//Try to get the thought
		JSONObject recThought = Record.getNode(thoughtId);

		if (recThought==null) {

			throw new Exception("Sorry, the thought "+thoughtId+" does not exist.");

		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("G")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (G). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("L")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (L). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("I")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (I). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("O")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (O). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("U")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (U). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("S")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (S). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("D")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (D). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("A")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (A). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("R")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (R). Please, delete the relations associated before.");
			
		}
		
		JSONObject ud = ((JSONObject) recThought.get("ud"));
		
		for (Object key : ud.keySet()) {
			
	        //based on you key types
	        String keyStr = (String)key;

	        String lang = keyStr.substring(0, 2);
	        keyStr = keyStr.substring(3, keyStr.length());
	        
			ExperienceManagerL1.deleteNodeTabLink(user, keyStr, thoughtId, lang);
			
			//Search in the word if there is thought
			JSONObject w = Record.getNode(keyStr);
			
			boolean b = true;
			for (Object keyW : ((JSONObject) w.get("fl")).keySet()) {

		        //based on you key types
		        String keyStrW = (String)keyW;

		        if (((JSONObject) ((JSONObject) w.get("fl")).get(keyStrW)).get("ftl")!=null) {
		        	
					b = false;
		        	break;
		        	
		        }

			}

			//Delete the word
			if (b) {
				
				keyStr = keyStr.substring(2, keyStr.length()-1);
				
				//Parse all letters in the word
				for(int i=0;i<keyStr.length();i++) {
					
					//Get the current letter
					String currentLetter = keyStr.substring(i, i+1);
					ExperienceManagerL1.deleteNodeTabLink(user, "S["+currentLetter+"]", "TL["+StringFx.str_to_hex(keyStr)+" "+i+" "+lang+"]", lang);
					
				}
				
				Record.remove(user, "W["+keyStr+"]");
				
			}
	        
		}
		
	}
	
	//Delete a thought
	public static void deleteLang(String user, String thoughtId, String searchLang) throws Exception {

		if (thoughtId.length()==0) {

			throw new Exception("Sorry, the thought id "+thoughtId+" is not valid [required].");

		}

		//Try to get the thought
		JSONObject recThought = Record.getNode(thoughtId);

		if (recThought==null) {

			throw new Exception("Sorry, the thought "+thoughtId+" does not exist.");

		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("G")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (G). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("L")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (L). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("I")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (I). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("O")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (O). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("U")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (U). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("S")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (S). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("R")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (R). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("A")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (A). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("R")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (R). Please, delete the relations associated before.");
			
		}
		
		JSONObject ud = ((JSONObject) recThought.get("ud"));
		
		for (Object key : ud.keySet()) {
			
	        //based on you key types
	        String keyStr = (String)key;

	        String lang = keyStr.substring(0, 2);
	        
	        if (lang.equals(searchLang)) {
	        
		        keyStr = keyStr.substring(3, keyStr.length());
		        
				ExperienceManagerL1.deleteNodeTabLink(user, keyStr, thoughtId, lang);
				
				//Search in the word if there is thought
				JSONObject w = Record.getNode(keyStr);
				
				boolean b = true;
				for (Object keyW : ((JSONObject) w.get("fl")).keySet()) {
	
			        //based on you key types
			        String keyStrW = (String)keyW;
	
			        if (((JSONObject) ((JSONObject) w.get("fl")).get(keyStrW)).get("ftl")!=null) {
			        	
						b = false;
			        	break;
			        	
			        }
	
				}
	
				//Delete the word
				if (b) {
					
					keyStr = keyStr.substring(2, keyStr.length()-1);
					
					//Parse all letters in the word
					for(int i=0;i<keyStr.length();i++) {
						
						//Get the current letter
						String currentLetter = keyStr.substring(i, i+1);
						ExperienceManagerL1.deleteNodeTabLink(user, "S["+currentLetter+"]", "TL["+StringFx.str_to_hex(keyStr)+" "+i+" "+lang+"]", lang);
			
					}
					
					Record.remove(user, "W["+keyStr+"]");
					
				}
				
			}
	        
		}
		
	}
	
	//Delete a thought
	public static void deleteWord(String user, String thoughtId, String searchWord) throws Exception {

		if (thoughtId.length()==0) {

			throw new Exception("Sorry, the thought id "+thoughtId+" is not valid [required].");

		}

		//Try to get the thought
		JSONObject recThought = Record.getNode(thoughtId);

		if (recThought==null) {

			throw new Exception("Sorry, the thought "+thoughtId+" does not exist.");

		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("G")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (G). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("L")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (L). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("I")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (I). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("O")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (O). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("U")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (U). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("S")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (S). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("D")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (D). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("A")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (A). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("R")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (R). Please, delete the relations associated before.");
			
		}
		
		JSONObject ud = ((JSONObject) recThought.get("ud"));
		
		for (Object key : ud.keySet()) {
			
	        //based on you key types
	        String keyStr = (String)key;

	        String lang = keyStr.substring(0, 2);
	        
	        keyStr = keyStr.substring(3, keyStr.length());
	        
	        if (keyStr.substring(2, keyStr.length()-1).equals(searchWord)) {
		        
				ExperienceManagerL1.deleteNodeTabLink(user, keyStr, thoughtId, lang);
				
				//Search in the word if there is thought
				JSONObject w = Record.getNode(keyStr);
				
				boolean b = true;
				for (Object keyW : ((JSONObject) w.get("fl")).keySet()) {
	
			        //based on you key types
			        String keyStrW = (String)keyW;
	
			        if (((JSONObject) ((JSONObject) w.get("fl")).get(keyStrW)).get("ftl")!=null) {
			        	
						b = false;
			        	break;
			        	
			        }
	
				}
	
				//Delete the word
				if (b) {
					
					keyStr = keyStr.substring(2, keyStr.length()-1);
					
					//Parse all letters in the word
					for(int i=0;i<keyStr.length();i++) {
						
						//Get the current letter
						String currentLetter = keyStr.substring(i, i+1);
						ExperienceManagerL1.deleteNodeTabLink(user, "S["+currentLetter+"]", "TL["+StringFx.str_to_hex(keyStr)+" "+i+" "+lang+"]", lang);
			
					}
					
					Record.remove(user, "W["+keyStr+"]");
					
				}
				
			}
	        
		}
		
	}
	
	//Delete a thought
	public static void delete(String user, String wordName, String thoughtId, String lang) throws Exception {

		if (wordName.length()==0) {

			throw new Exception("Sorry, the word "+wordName+" is not valid [required].");

		}

		if (thoughtId.length()==0) {

			throw new Exception("Sorry, the thought id "+thoughtId+" is not valid [required].");

		}

		//Try to get the record
		JSONObject rec = Record.getNode("W["+wordName+"]");

		if (rec==null) {

			throw new Exception("Sorry, the word "+wordName+" does not exist.");

		}

		//Try to get the thought
		JSONObject recThought = Record.getNode(thoughtId);

		if (recThought==null) {

			throw new Exception("Sorry, the thought "+thoughtId+" does not exist.");

		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("G")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (G). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("L")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (L). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("I")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (I). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("O")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (O). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("U")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (U). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("S")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (S). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("D")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (D). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("A")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (A). Please, delete the relations associated before.");
			
		}
		
		if (((JSONObject) ((JSONObject) recThought.get("fl")).get("R")).get("ftl")!=null) {
			
			throw new Exception("Sorry, the thought <"+thoughtId+"> cannot be deleted due to under tab links (R). Please, delete the relations associated before.");
			
		}

		ExperienceManagerL1.deleteNodeTabLink(user, "W["+wordName+"]", thoughtId, lang);
		
		//Search in the word if there is thought
		String keyStr = "W["+wordName+"]";
		JSONObject w = Record.getNode(keyStr);
		
		boolean b = true;
		for (Object keyW : ((JSONObject) w.get("fl")).keySet()) {

	        //based on you key types
	        String keyStrW = (String)keyW;

	        if (((JSONObject) ((JSONObject) w.get("fl")).get(keyStrW)).get("ftl")!=null) {
	        	
				b = false;
	        	break;
	        	
	        }

		}

		//Delete the word
		if (b) {
			
			keyStr = keyStr.substring(2, keyStr.length()-1);
			
			//Parse all letters in the word
			for(int i=0;i<keyStr.length();i++) {
				
				//Get the current letter
				String currentLetter = keyStr.substring(i, i+1);
				ExperienceManagerL1.deleteNodeTabLink(user, "S["+currentLetter+"]", "TL["+StringFx.str_to_hex(keyStr)+" "+i+" "+lang+"]", lang);
	
			}
			
			Record.remove(user, "W["+keyStr+"]");
			
		}

	}

}
