/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.session;

import java.io.IOException;
import java.util.Map.Entry;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;

import re.jpayet.mentdb.MentDBConnector;
import re.jpayet.mentdb.core.basic.BasicResult;
import re.jpayet.mentdb.core.command.Command;
import re.jpayet.mentdb.core.command.MQLValue;
import re.jpayet.mentdb.core.user.UserManager;
import re.jpayet.mentdb.tools.Misc;

//The session
public class Session {
	
	//Properties
	public static ConcurrentHashMap<String, WebSocketThread> allSessions = new ConcurrentHashMap<String, WebSocketThread>();
	public static ConcurrentHashMap<String, MentDBConnector> allConnections = new ConcurrentHashMap<String, MentDBConnector>();

	//Send target to me and another
	public static void sendTargetToMeAndAnother(String user, String target, int type, String value) {
		
		try {
			
			String[] allUsers = Misc.JSONArray2StringTab(UserManager.showAll("mentdb"));
			
			//Initialization
			BasicResult br = new BasicResult(user, "ai", type, value, allUsers, WebSocketThread.target.get(user), UserManager.allUsersWhoTalkingWith(user), 0);
			
			Session.allSessions.get(user).session.getRemote().sendString(br.toString());

			br = new BasicResult(target, "ai", type, value, allUsers, WebSocketThread.target.get(target), UserManager.allUsersWhoTalkingWith(target), 0);
			
			Session.allSessions.get(target).session.getRemote().sendString(br.toString());
			
		} catch (Exception f) {};
		
	}
	
	//Send message to me
	public static void sendMessageToMe(String user, int type, String value) {
		
		try {
			
			String[] allUsers = Misc.JSONArray2StringTab(UserManager.showAll("mentdb"));
			
			//Initialization
			BasicResult br = new BasicResult(user, "ai", type, value, allUsers, WebSocketThread.target.get(user), UserManager.allUsersWhoTalkingWith(user), 0);
			
			Session.allSessions.get(user).session.getRemote().sendString(br.toString());

		} catch (Exception f) {};
		
	}

	//Send a message to other users
	public static void sendMessageToOthers(String user, int type, String value) throws Exception {
		
		String[] allUsers = Misc.JSONArray2StringTab(UserManager.showAll("mentdb"));
		
		//Parse all connected users
		for (Entry<String, WebSocketThread> e : Session.allSessions.entrySet()) {
			
			try {
				
				//All excepted me
				if (!e.getKey().equals(user)) {
					
					//Initialization
					BasicResult br = new BasicResult(e.getKey(), "ai", type, value, allUsers, WebSocketThread.target.get(e.getKey()), UserManager.allUsersWhoTalkingWith(e.getKey()), 0);
					
					e.getValue().session.getRemote().sendString(br.toString());
				
				}
				
			} catch (Exception f) {};
			
		}
		
	}

	//Ai send a message to me
	public static void aiSendMessageToMe(String user, int type, String value) throws Exception {
		
		String[] allUsers = Misc.JSONArray2StringTab(UserManager.showAll("mentdb"));
		
		//Parse all connected users
		for (Entry<String, WebSocketThread> e : Session.allSessions.entrySet()) {
			
			try {
				
				//Only me
				if (e.getKey().equals(user)) {
					
					//Initialization
					BasicResult br = new BasicResult(e.getKey(), "ai", type, value, allUsers, WebSocketThread.target.get(e.getKey()), UserManager.allUsersWhoTalkingWith(e.getKey()), 0);
					
					e.getValue().session.getRemote().sendString(br.toString());
				
				}
				
			} catch (Exception f) {};
			
		}
		
	}

	//MentDB send a message to me
	public static void mentdbSendMessageToMe(String user, int type, String value, long seconds) throws Exception {
		
		String[] allUsers = Misc.JSONArray2StringTab(UserManager.showAll("mentdb"));
		
		//Parse all connected users
		for (Entry<String, WebSocketThread> e : Session.allSessions.entrySet()) {
			
			try {
				
				//Only me
				if (e.getKey().equals(user)) {
					
					//Initialization
					BasicResult br = new BasicResult(e.getKey(), "mentdb", type, value, allUsers, WebSocketThread.target.get(e.getKey()), UserManager.allUsersWhoTalkingWith(e.getKey()), seconds);
					
					e.getValue().session.getRemote().sendString(br.toString());
				
				}
				
			} catch (Exception f) {};
			
		}
		
	}
	
	//Send a message to all target and me
	public static void sendMessageToTargetsAndMe(String user, int type, String inputText, Vector<Vector<MQLValue>> inputVector) {
		
		//Initialization
		Vector<String> target = WebSocketThread.target.get(user);
		
		//Send to the current user
		try {

			BasicResult br = new BasicResult(user, user, type, inputText, 0);
			if (Session.allSessions.get(user).mqlQueryMode && !inputText.equals("graph symbol \"ai\";") && !inputText.startsWith("refresh ") && !inputText.startsWith("in editor ")) 
				Session.allSessions.get(user).session.getRemote().sendString(br.toString());
			else if (!Session.allSessions.get(user).mqlQueryMode) Session.allSessions.get(user).session.getRemote().sendString(br.toString());
			
		} catch (Exception f) {};
		
		//Send to all target except ai
		for (int i=0;i<target.size();i++) {
				
			if (!target.elementAt(i).equals("ai")) {
				
				try {

					BasicResult br = new BasicResult(target.elementAt(i), user, type, inputText, 0);
					Session.allSessions.get(target.elementAt(i)).session.getRemote().sendString(br.toString());
				
				} catch (Exception f) {};
			
			}
			
		}
		
		//Send to ai
		try {
			
			if (target.contains("ai")) {
				
				Command.execute(user, inputText, inputVector);
			
			}
			
		} catch (Exception f) {};
		
	}
	
	//Send a message to all users
	public static void aiSendMessage(int type, String value) {
		
		//Parse all connected users
		for (Entry<String, WebSocketThread> e : Session.allSessions.entrySet()) {
			
			try {
				
				//Initialization
				BasicResult br = new BasicResult("", "ai", type, value, 0);
				
				e.getValue().session.getRemote().sendString(br.toString());
				
			} catch (Exception f) {};
			
		}
		
	}
	
	//Close a session
	public static void close(String user) throws IOException {
		
		allSessions.get(user).session.close(0, "exit");
		
	}

}