/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.session.env;

import java.util.HashMap;

import org.json.simple.JSONObject;

//The environment variable
public class EnvManager {

	//The environment variable
	public JSONObject varEnv = new JSONObject();
	public HashMap<String, Object> jsonObj = new HashMap<String, Object>();
	
	//Show the environment
	public JSONObject show() {
		
		return varEnv;
		
	}

	//Check if a string is a valid variable name
	public static boolean is_valid_varname(String varName) throws Exception {
		
		//Generate an error if the variable name is not valid
		if (!varName.startsWith("[") || !varName.endsWith("]") || varName.length()<3) {
			
			return false;
		
		} else {
			
			return true;
		
		}
		
	}

	//Set variable
	@SuppressWarnings("unchecked")
	public void set(String varName, String value) throws Exception {
		
		//Generate an error if the variable name is not valid
		if (!is_valid_varname(varName)) {
			throw new Exception("Sorry, the variable name "+varName+" is not valid (example: [var1]).");
		}
		
		varEnv.put(varName, value);
		
	}

	//Increment variable
	@SuppressWarnings("unchecked")
	public String incr(String varName, String i) throws Exception {
		
		//Generate an error if the variable does not exist
		if (!varEnv.containsKey(varName)) {
			throw new Exception("Sorry, the variable name "+varName+" does not exist.");
		}
		
		try {
			
			double d = Double.parseDouble(varEnv.get(varName)+"")+Double.parseDouble(i);
			String result = ""+d;
			if (result.endsWith(".0")) result = result.substring(0, result.length()-2);
			
			varEnv.put(varName, result);
			return result;
			
		} catch (Exception e) {
			throw new Exception("Sorry, cannot increment the variable "+varName+" ["+e.getMessage()+"].");
		}
		
	}

	//Decrement variable
	@SuppressWarnings("unchecked")
	public String decr(String varName, String i) throws Exception {
		
		//Generate an error if the variable does not exist
		if (!varEnv.containsKey(varName)) {
			throw new Exception("Sorry, the variable name "+varName+" does not exist.");
		}
		
		try {
			
			double d = Double.parseDouble(varEnv.get(varName)+"")-Double.parseDouble(i);
			String result = ""+d;
			if (result.endsWith(".0")) result = result.substring(0, result.length()-2);
			
			varEnv.put(varName, result);
			return result;
			
		} catch (Exception e) {
			throw new Exception("Sorry, cannot decrement the variable "+varName+" ["+e.getMessage()+"].");
		}
		
	}

	//remove variable
	public void remove(String varName) throws Exception {
		
		//Generate an error if the variable does not exist
		if (!varEnv.containsKey(varName)) {
			throw new Exception("Sorry, the variable name "+varName+" does not exist.");
		}
		
		varEnv.remove(varName);
		
	}

	//exist variable
	public boolean exist(String varName) {
		
		return varEnv.containsKey(varName);
		
	}

	//get variable
	public String get(String varName) throws Exception {
		
		//Generate an error if the variable does not exist
		if (!varEnv.containsKey(varName)) {
			throw new Exception("Sorry, the variable name "+varName+" does not exist.");
		}
		
		return (String) varEnv.get(varName);
		
	}

}