/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.user;

import java.io.IOException;
import java.util.Set;
import java.util.Vector;
import java.util.Map.Entry;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

import re.jpayet.mentdb.MentDBConnector;
import re.jpayet.mentdb.core.brain.Brain;
import re.jpayet.mentdb.core.db.Record;
import re.jpayet.mentdb.core.db.file.experience.ExperienceManagerL2;
import re.jpayet.mentdb.core.db.file.json.JsonManager;
import re.jpayet.mentdb.core.db.file.node.NodeManager;
import re.jpayet.mentdb.core.entity.concentration.ConcentrationManager;
import re.jpayet.mentdb.core.entity.link.LinkManager;
import re.jpayet.mentdb.core.entity.relation.RelationManager;
import re.jpayet.mentdb.core.entity.thought.ThoughtManager;
import re.jpayet.mentdb.core.entity.word.WordManager;
import re.jpayet.mentdb.core.fx.StringFx;
import re.jpayet.mentdb.core.session.Session;
import re.jpayet.mentdb.core.session.WebSocketThread;
import re.jpayet.mentdb.server.Start;
import re.jpayet.mentdb.tools.Misc;

//User management
public class UserManager {
	
	public static String firstUser = "";
	
	//Stimulate a user
	public static void stimulate(String cuser, String login) throws Exception {
		
		//Get the user node
		JSONObject rec = Record.getNode("U["+login+"]");
		if (rec==null) {
			throw new Exception("Sorry, the login "+login+" does not exist."); 
		}
		
		RelationManager.stimulate(cuser, ""+rec.get("ru"), true);
		
		firstUser = login;
		
	}
	
	//disable a user
	public static void disable(String cuser, String login) throws Exception {
		
		if (login.equals("admin") || login.equals("ai")) {
			
			throw new Exception("Sorry, 'admin' or 'ai' cannot be disabled."); 
			
		}
		
		JSONArray groups = GroupManager.userGetGroup(login);
		
		for(int i=0;i<groups.size();i++) {
			
			GroupManager.ungrantUser(cuser, groups.get(i)+"", login);
			
		}
		
	}
	
	//Get a user from his token
	public static String get_user_from_token(String token) throws Exception{
		
		//Initialization
		
		JSONObject rec = Record.getNode("UT["+token+"]");
		
		if (rec==null) {
			
			throw new Exception("Sorry, the token '"+token+"' does not exist."); 
			
		} else {
			
			return rec.get("login")+"";
			
		}
		
	}
	
	//Get a token from user
	public static String get_token_from_user(String user) throws Exception{
		
		//Initialization
		
		JSONObject rec = Record.getNode("U["+user+"]");
		
		if (rec==null) {
			
			throw new Exception("Sorry, the user '"+user+"' does not exist."); 
			
		} else {
			
			return rec.get("token")+"";
			
		}
		
	}
	
	//Reset token for a specific user
	@SuppressWarnings("unchecked")
	public static void reset_token(String cuser, String login) throws Exception{
		
		//Initialization
		JSONObject rec = Record.getNode("U["+login+"]");
		
		if (rec==null) {
			
			throw new Exception("Sorry, the user '"+login+"' does not exist."); 
			
		} else {
			
			String tok = StringFx.generate_random_str("70");

			String lastToken = rec.get("token")+"";
			rec.put("token", tok);
			
			Record.update(cuser, "U["+login+"]", rec.toJSONString());
			
			Record.remove(cuser, "UT["+lastToken+"]");
			
			JSONObject token = new JSONObject();
			token.put("login", login);
			Record.add(cuser, "UT["+tok+"]", token.toJSONString());
			
		}
		
	}
	
	//Reset all tokens
	public static void reset_tokens(String cuser) throws Exception{
		
		//Initialization
		JSONArray users = UserManager.showAll(cuser);
		
		for(int i=0;i<users.size();i++) {
			
			reset_token(cuser, users.get(i)+"");
			
		}
		
	}
	
	//Show first user
	public static String first(String cuser) throws Exception {
		
		if (firstUser.equals("")) {
		
			//Initialization
			String tl = ExperienceManagerL2.getFirstTabLink(cuser, "["+LinkManager.get_value("@[r_fr_?_est_un_utilisateur]")+" 2/3]", "L");
			
			//Get the current user relation
			String userRelationTabLink = tl;
			
			String relationId = Misc.atom(userRelationTabLink, 1, " ").substring(1);
			
			String thoughtId = ""+RelationManager.showThoughtNodes(cuser, relationId).get(0);
			
			JSONObject bd = Record.getNode(thoughtId);
			
			firstUser = ""+bd.get("login");
			
		}
		
		return firstUser;
		
	}
	
	//Show get user
	public static String get(String cuser, int position) throws Exception {
		
		//Initialization
		String u = "";
		
		if (position>=ConcentrationManager.getConcentrationDepth("C[user]")) {

			throw new Exception("Sorry, the position > concentration depth.");

		}
		
		JSONArray users = RelationManager.showTabLinks(cuser, "["+LinkManager.get_value("@[r_fr_?_est_un_utilisateur]")+" 2/3]", "L", ConcentrationManager.getConcentrationDepth("C[relation]"));
		
		//Parse all user relation tab links
		for(int i=0;i<users.size();i++) {
			
			if (i==ConcentrationManager.getConcentrationDepth("C[user]")) break; 
			
			//Get the current user relation
			String userRelationTabLink = users.get(i)+"";
			
			String relationId = Misc.atom(userRelationTabLink, 1, " ").substring(1);
			
			String thoughtId = ""+RelationManager.showThoughtNodes(cuser, relationId).get(0);
			
			JSONObject bd = Record.getNode(thoughtId);
			
			u = bd.get("login")+"";
			
			if (i==position) break;
			
		}
		
		return u;
		
	}
	
	//Show all users
	@SuppressWarnings("unchecked")
	public static JSONArray showAll(String cuser) throws Exception {
		
		//Initialization
		JSONArray result = new JSONArray();
		
		JSONArray users = RelationManager.showTabLinks(cuser, "["+LinkManager.get_value("@[r_fr_?_est_un_utilisateur]")+" 2/3]", "L", ConcentrationManager.getConcentrationDepth("C[relation]"));
		
		//Parse all user relation tab links
		for(int i=0;i<users.size();i++) {
			
			if (i==ConcentrationManager.getConcentrationDepth("C[user]")) break; 
			
			//Get the current user relation
			String userRelationTabLink = users.get(i)+"";
			
			String relationId = Misc.atom(userRelationTabLink, 1, " ").substring(1);
			
			String thoughtId = ""+RelationManager.showThoughtNodes(cuser, relationId).get(0);
			
			JSONObject bd = Record.getNode(thoughtId);
			
			String login = ""+bd.get("login");
			
			if (GroupManager.userGetGroup(login).size()>0) {
				result.add(login);
			}
			
		}
		
		return result;
		
	}
	
	//Create a new user
	@SuppressWarnings("unchecked")
	public static void create(String cuser, String login, String separator, String password, boolean force) throws Exception {
		
		//Check parameters
		if (Misc.lrtrim(login).equals("")) {
			throw new Exception("Sorry, the login is required."); 
		}
		
		if (Misc.lrtrim(password).equals("")) {
			throw new Exception("Sorry, the password is required."); 
		}
		
		login = login.toLowerCase();

		if (login==null || login.equals("")) {
			throw new Exception("Sorry, the login is required."); 
		}
		
		//Get the user node
		JSONObject rec = Record.getNode("U["+login+"]");
		if (rec!=null) {
			throw new Exception("Sorry, the login "+login+" already exist."); 
		}
		
		//Create the user group
		if (!GroupManager.exist(login)) GroupManager.add(cuser, login);
		else {
			if (!force) throw new Exception("Sorry, the group '"+login+"' already exist. Use force option to create the user ...");
		}
		
		//Add the user thought
		String userThought = "";
		
		if (!WordManager.exist(cuser, login.toLowerCase(), Start.FIRST_LANGUAGE)) {
			userThought = WordManager.create(cuser, login.toLowerCase(), separator, Start.FIRST_LANGUAGE);
		} else {
			userThought = ThoughtManager.create(cuser, login.toLowerCase(), separator, Start.FIRST_LANGUAGE);
		}

		JSONObject userDirectAccess = new JSONObject();
		userDirectAccess.put("th", userThought);
		
		userDirectAccess.put("lw", new JSONArray());
		
		password = Misc.md5(password);
		
		//Add the password
		NodeManager.oinsert(cuser, userThought, "/", "login", login, "STR");
		NodeManager.oinsert(cuser, userThought, "/", "password", password, "STR");
		NodeManager.oinsert(cuser, userThought, "/", "process_tab", "[]", "ARRAY");
		
		//Create the relation someone is a user
		String userRelation = RelationManager.createSubWithPosition(cuser, LinkManager.get_value("@[r_fr_?_est_un_utilisateur]"), userThought+" "+LinkManager.get_value("@[th_fr_est]")+" "+LinkManager.get_value("@[r_fr_un_utilisateur]"), "1 2 3", Start.FIRST_LANGUAGE, "L");
		userDirectAccess.put("ru", userRelation);
		
		JSONObject objFl = new JSONObject();
		objFl.put("ftl", null); //First tab link
		objFl.put("ltl", null); //Last tab link
		
		userDirectAccess.put("fl", objFl);
		String tok = StringFx.generate_random_str("70");
		userDirectAccess.put("token", tok);

		Record.add(cuser, "U["+login+"]", userDirectAccess.toJSONString());
		
		JSONObject token = new JSONObject();
		token.put("login", login);
		Record.add(cuser, "UT["+tok+"]", token.toJSONString());
		
		firstUser = login;
		
		//Grant the user to his group
		if (!GroupManager.isGrantedUser(cuser, login, login)) {
			
			GroupManager.grantUser(cuser, login, login);
		
		}
		
	}
	
	//Check the password of a specific user
	public static boolean checkPassword(String login, String password) {
		
		try {
			
			//Get the user node
			JSONObject bd = Record.getNode("U["+login.toLowerCase()+"]");
			bd = Record.getNode(bd.get("th")+"");
			
			if ((bd.get("password")+"").equals(Misc.md5(password))) return true;
			else return false;
			
		} catch (Exception e) {
			return false;
		}
		
	}
	
	//Load the user process tab into the brain
	@SuppressWarnings("unchecked")
	public static void loadProcessTab(String login) throws Exception {
		
		Brain.stimulation.put(login, (JSONArray) JsonManager.load(NodeManager.select(UserManager.getThought(login), "process_tab")));
		
	}
	
	//Load the user process tab into the brain
	public static void unloadProcessTab(String login) throws IOException {
		
		if (Brain.stimulation.containsKey(login)) {
			Brain.stimulation.remove(login);
		}
		
	}
	
	//Get the user thought
	public static String getThought(String login) throws IOException {
		
		//Get the user node
		JSONObject bd = Record.getNode("U["+login.toLowerCase()+"]");
		return bd.get("th")+"";
		
	}
	
	//Update the password of a specific user
	@SuppressWarnings("unchecked")
	public static void updatePassword(String cuser, String login, String password) throws Exception {
		
		//Check parameters
		if (Misc.lrtrim(login).equals("")) {
			throw new Exception("Sorry, the login is required."); 
		}
		
		if (Misc.lrtrim(password).equals("")) {
			throw new Exception("Sorry, the password is required."); 
		}
		
		login = login.toLowerCase();
		
		//Get the user node
		JSONObject rec = Record.getNode("U["+login+"]");
		if (rec==null) {
			throw new Exception("Sorry, the user "+login+" does not exist."); 
		}
		
		String thoughtId = rec.get("th")+"";
		rec = Record.getNode(thoughtId);
		
		rec.put("password", Misc.md5(password));
		
		Record.update(cuser, thoughtId, rec.toJSONString());
		
	}
	
	//Check if a user already exist
	public static boolean exist(String login) throws IOException {
		
		if (Record.getNode("U["+login+"]")==null) {
			return false;
		} else {
			return true;
		}
		
	}
	
	//Connect a user
	public static synchronized boolean connect(String user, String password, String mode) throws Exception {
		
		if (Session.allSessions.containsKey(user)) {

			if (mode.equals("WS")) throw new Exception("j12hki95orm35hrm62vni90tkmr33sdy4");
			else throw new Exception("User already connected.");
			
		}
		
		//Connect to the database
		MentDBConnector mentdb = new MentDBConnector("127.0.0.1", Start.SERVER_PORT, Start.SESSION_CONNECT_TIMEOUT, Start.SESSION_TIMEOUT);
		
		if (user.equals("ai")) {
			
			if (mode.equals("WS")) throw new Exception("jajidfm62mr7i8dtyfr2tyrypzea8tyyoejht6");
			else throw new Exception("\"ai\" is a system user.");
			
		} else if (mentdb.connect(user, password)) {
			
			Session.allConnections.put(user, mentdb);
			return true;

		} else {
			
			if (mode.equals("WS")) throw new Exception(mentdb.serverConnectionStateId);
			else throw new Exception(mentdb.clientConnectionState);
			
		}
		
	}
	
	//Disconnect a user
	public static void disconnect(String user) {
		
		try {
			
			if (Session.allConnections.containsKey(user)) {
				
				try {Session.allConnections.get(user).close();} catch (Exception e) {};
				try {Session.allConnections.remove(user);} catch (Exception e) {};

			}
		} catch (Exception e) {};
		
	}
	
	//Get all user who talking with a specific user
	public static Vector<String> allUsersWhoTalkingWith(String user) {
		
		//Initialization
		Set<Entry<String, WebSocketThread>> set = Session.allSessions.entrySet();
		Vector<String> result = new Vector<String>();
		
		//Parse all connected users
		for (Entry<String, WebSocketThread> e : set) {
			
			try {
				
				//Get the user
				String username = e.getKey();
				
				if (WebSocketThread.target.get(username).contains(user)) {
					
					result.add(username);
				
				}
			
			} catch (Exception f) {};
			
		}
		
		return result;
		
	}
	
}
