/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.server;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.lang.management.ManagementFactory;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;

import javax.servlet.http.HttpSessionEvent;
import javax.servlet.http.HttpSessionListener;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.ContextHandlerCollection;
import org.eclipse.jetty.server.handler.ErrorHandler;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;
import org.eclipse.jetty.webapp.WebAppContext;
import org.eclipse.jetty.websocket.servlet.WebSocketServlet;
import org.eclipse.jetty.websocket.servlet.WebSocketServletFactory;

import re.jpayet.mentdb.core.brain.Brain;
import re.jpayet.mentdb.core.db.Database;
import re.jpayet.mentdb.core.db.Record;
import re.jpayet.mentdb.core.db.file.data.DataFile;
import re.jpayet.mentdb.core.db.file.data.DataFilePool;
import re.jpayet.mentdb.core.db.file.index.IndexFile;
import re.jpayet.mentdb.core.db.file.index.IndexFilePool;
import re.jpayet.mentdb.core.db.file.transaction.Transaction;
import re.jpayet.mentdb.core.entity.concentration.ConcentrationManager;
import re.jpayet.mentdb.core.entity.data_type.DataTypeManager;
import re.jpayet.mentdb.core.entity.language.LanguageManager;
import re.jpayet.mentdb.core.entity.reader_type.ReaderTypeManager;
import re.jpayet.mentdb.core.fx.DateFx;
import re.jpayet.mentdb.core.fx.StringFx;
import re.jpayet.mentdb.core.job.JobManager;
import re.jpayet.mentdb.core.log.Log;
import re.jpayet.mentdb.core.parameter.ParameterManager;
import re.jpayet.mentdb.core.session.SessionThread;
import re.jpayet.mentdb.core.session.SessionThreadAgent;
import re.jpayet.mentdb.core.session.WebSocketThread;
import re.jpayet.mentdb.core.stat.Statistic;
import re.jpayet.mentdb.core.user.UserManager;
import re.jpayet.mentdb.doc.MentDBFeature;
import re.jpayet.mentdb.doc.MQLDocumentation;
import re.jpayet.mentdb.tools.Misc;

//MentDB Server
public class Start
{
	
	//To change the MentDB Server version
	static public String version = "alpha-0.3.1";
	static public String copyright = "2012-"+thisYear();
	static public boolean newDatabase = false;
	
	//Initialization
	public static Database db = null;
	public static ServerSocket serverSocket = null;
	public static boolean listening = true;
	public static long exceededSessions = 0;
	public static Server webServer = null;
	public static Brain brain = null;
	
	//Configuration
	static public String REMOTE_ACCESS = "DENIED";
	static public int SERVER_PORT = 0;
	static public int SESSION_TIMEOUT = 0;
	static public int SESSION_CONNECT_TIMEOUT = 0;
	static public int NB_INDEX_FILE = 0;
	static public int MAX_SESSION = 0;
	static public int WEB_SERVER_PORT = 0;
	static public String FIRST_LANGUAGE = "";
	static public String AI_FIRST_NAME = "";
	static public String AI_LAST_NAME = "";
	static public int CHECK_DOCUMENTATION = 0;
	
	//Get the current year
	public static String thisYear() {
		
		try {
			return DateFx.year(DateFx.curdate());
		} catch (Exception e) {
			return "2050";
		}
		
	}
	
	//Constructor
	public Start() {

		System.out.println("/////////////////////////////////////////////////////////////////");
		System.out.println("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
		System.out.println("+ Mentalese Database Engine (MentDB)              v_"+Start.version+" +");
		System.out.println("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
		System.out.println("-----------------------------------------------------------------");

		//Display MentDB information
		Log.write("/////////////////////////////////////////////////////////////////");
		Log.write("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
		Log.write("+ Mentalese Database Engine (MentDB)              v_"+Start.version+" +");
		Log.write("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
		Log.write("-----------------------------------------------------------------");
		
		//Load from the config file
		Misc.system_out_print("Loading global config from 'server.conf' > ", false, "", "");
		
		CHECK_DOCUMENTATION = Integer.parseInt(Misc.conf_value("conf"+File.separator+"server.conf", "DOCUMENTATION", "check_documentation"));
		MAX_SESSION = Integer.parseInt(Misc.conf_value("conf"+File.separator+"server.conf", "DATABASE", "MAX_SESSION"));
		SESSION_TIMEOUT = Integer.parseInt(Misc.conf_value("conf"+File.separator+"server.conf", "DATABASE", "SESSION_TIMEOUT"));
		SESSION_CONNECT_TIMEOUT = Integer.parseInt(Misc.conf_value("conf"+File.separator+"server.conf", "DATABASE", "SESSION_CONNECT_TIMEOUT"));
		SERVER_PORT = Integer.parseInt(Misc.conf_value("conf"+File.separator+"server.conf", "DATABASE", "SERVER_PORT"));
		NB_INDEX_FILE = Integer.parseInt(Misc.conf_value("conf"+File.separator+"server.conf", "DATABASE", "NB_INDEX_FILE"));
		IndexFilePool.NB_INDEX_FILE_OPEN = Integer.parseInt(Misc.conf_value("conf"+File.separator+"server.conf", "DATABASE", "NB_INDEX_FILE_OPEN"));
		DataFilePool.NB_DATA_FILE_OPEN = Integer.parseInt(Misc.conf_value("conf"+File.separator+"server.conf", "DATABASE", "NB_DATA_FILE_OPEN"));
		DataFile.BLOCK_SIZE = Integer.parseInt(Misc.conf_value("conf"+File.separator+"server.conf", "DATABASE", "DATA_BLOCK_SIZE"));
		DataFile.BLOCK_SIZE_ADD_TO_BLOCK = DataFile.BLOCK_SIZE+DataFile.ADD_TO_BLOCK;
		DataFile.EMPTY_BLOCK = StringUtils.repeat(" ", DataFile.BLOCK_SIZE);
		DataFile.MAX_BLOCK = Long.parseLong(Misc.conf_value("conf"+File.separator+"server.conf", "DATABASE", "MAX_DATA_BLOCK_SIZE"));
		Record.RECORD_CACHE_SIZE = Integer.parseInt(Misc.conf_value("conf"+File.separator+"server.conf", "DATABASE", "RECORD_CACHE_SIZE"));
		Start.FIRST_LANGUAGE = Misc.conf_value("conf"+File.separator+"server.conf", "DATABASE", "FIRST_LANGUAGE");
		Start.AI_FIRST_NAME = Misc.conf_value("conf"+File.separator+"server.conf", "AI", "FIRSTNAME");
		Start.AI_LAST_NAME = Misc.conf_value("conf"+File.separator+"server.conf", "AI", "LASTNAME");
		REMOTE_ACCESS = Misc.conf_value("conf"+File.separator+"server.conf", "DATABASE", "REMOTE_ACCESS");
		WEB_SERVER_PORT = Integer.parseInt(Misc.conf_value("conf"+File.separator+"server.conf", "WEBSERVER", "WEB_SERVER_PORT"));
		
		Misc.system_out_print("[OK]", true, "", "");
		Log.write("Loading global config from 'server.conf' > OK");
		
		try {
			
			//Allow remote connections or denied
			if (REMOTE_ACCESS.equals("ALLOW")) {
				serverSocket = new ServerSocket(Start.SERVER_PORT);
			}
			else serverSocket = new ServerSocket(Start.SERVER_PORT, 0, InetAddress.getByName(null));
			
			//Check if create a new database
			if (!(new File("data").exists())) {
				
				Start.newDatabase = true;
			
			}
			
			if (newDatabase) Misc.system_out_print("Create the database > ", false, "", "");
			else Misc.system_out_print("Open the database > ", false, "", "");
			
			//Open the database
			db = new Database();
			db.open();
			
			if (newDatabase) Misc.system_out_print("[OK]", true, "", "100/100%");
			else Misc.system_out_print("[OK]", true, "", "");
			
			if (newDatabase) Log.write("Create database > [OK]");
			else Log.write("Open database > [OK]");
			
			Misc.system_out_print("Rollback last transactions > ", false, "", "");
			
			//Roll back
			String[] userTransactions = (new File("data"+File.separator+"transaction")).list();
			//Parse all transactions
			for(int itransaction = 0;itransaction<userTransactions.length;itransaction++) {
				if ((new File("data"+File.separator+"transaction"+File.separator+userTransactions[itransaction]+File.separator+"rollback.log")).exists()) {
					//Open the log file
					Transaction.logFiles.put(userTransactions[itransaction], new RandomAccessFile("data"+File.separator+"transaction"+File.separator+userTransactions[itransaction]+File.separator+"rollback.log", "rw"));
					Transaction.rollback(userTransactions[itransaction]);
				}
			}
			
			Misc.system_out_print("[OK]", true, "", "");
			
			Log.write("Rollback last transactions > [OK]");
			
			Misc.system_out_print("Start the heartbeat > ", false, "", "");

			Statistic.startHeartbeat10m();
			Statistic.startHeartbeat24h();
			
			Misc.system_out_print("[OK]", true, "", "");
			
			Log.write("Start the heartbeat > [OK]");
			
			Misc.system_out_print("Resize the index file > ", false, "", "");
			
			//Resize
			IndexFile.resize("mentdb", null);
			
			//Commit all transactions
			for(int itransaction = 0;itransaction<userTransactions.length;itransaction++) {
				if ((new File("data"+File.separator+"transaction"+File.separator+userTransactions[itransaction]+File.separator+"rollback.log")).exists()) {
					Transaction.commit(userTransactions[itransaction]);
					if (!userTransactions[itransaction].equals("mentdb")) {
						Transaction.logFiles.remove(userTransactions[itransaction]);
						Misc.deleteFile("data"+File.separator+"transaction"+File.separator+userTransactions[itransaction]);
						
					}
				}
			}
			
			Misc.system_out_print("[OK]", true, "", "");
			
			Log.write("Resize the index file > [OK]");
			
			if (newDatabase) {

				System.out.println("-----------------------------------------------------------------");
				System.out.println("DATABASE INITIALIZATION                                      [..]");
				System.out.println("BEGIN -----------------------------------------------------------");
				
				//Initialization
				Database.init("mentdb");

				System.out.println("END -------------------------------------------------------------");
				
				Log.write("Database initialization > [OK]");
				
			} else {
			
				Misc.system_out_print("Load global parameters > ", false, "", "");
				
				//Load the global parameter
				ParameterManager.load();
				
				Misc.system_out_print("[OK]", true, "", "");
				
				Log.write("Load global parameters > [OK]");
			
				Misc.system_out_print("Load concentration > ", false, "", "");
				
				//Load the concentration depth
				ConcentrationManager.load("mentdb");
				
				Misc.system_out_print("[OK]", true, "", "");
				
				Log.write("Load concentration > [OK]");
			
				Misc.system_out_print("Load data type > ", false, "", "");
				
				//Load data type
				DataTypeManager.reload("mentdb");
				
				Misc.system_out_print("[OK]", true, "", "");
				
				Log.write("Load data type > [OK]");
			
				Misc.system_out_print("Load reader type > ", false, "", "");
				
				//Load reader type
				ReaderTypeManager.reload("mentdb");
				
				Misc.system_out_print("[OK]", true, "", "");
				
				Log.write("Load reader type > [OK]");
				
			}
			
			Misc.system_out_print("Load index files > ", false, "", "");
			for(int i=0;i<IndexFilePool.NB_INDEX_FILE_OPEN;i++) {
				IndexFilePool.get(i);
			}
			Misc.system_out_print("[OK]", true, "", "");
			Log.write("Load index files > [OK]");
			
			//Update the admin password
			UserManager.updatePassword("mentdb", "admin", Misc.conf_value("conf"+File.separator+"server.conf", "ADMIN", "PWD"));
			
			//Starting the web server
			Misc.system_out_print("Starting the web server ("+WEB_SERVER_PORT+") > ", false, "", "");
			webServer = new Server(WEB_SERVER_PORT);
			String[] configuration = new String[] {
		    	     "org.eclipse.jetty.webapp.WebInfConfiguration",
		    	     "org.eclipse.jetty.webapp.WebXmlConfiguration",
		    	     "org.eclipse.jetty.webapp.MetaInfConfiguration",
		    	     "org.eclipse.jetty.webapp.FragmentConfiguration",
		    	     "org.eclipse.jetty.plus.webapp.EnvConfiguration",
		    	     "org.eclipse.jetty.plus.webapp.PlusConfiguration",
		    	     "org.eclipse.jetty.annotations.AnnotationConfiguration",
		    	     "org.eclipse.jetty.webapp.JettyWebXmlConfiguration" };
	        ContextHandlerCollection contexts = new ContextHandlerCollection();
	        WebAppContext webappdocroot = new WebAppContext();
	        webappdocroot.setWelcomeFiles(new String[] { 
			        "index.html", "index.jsp"
			    });
	        webappdocroot.setAttribute("org.eclipse.jetty.webapp.configuration", configuration);
	        webappdocroot.setResourceBase("docs");
	        webappdocroot.setContextPath("/");
	        webappdocroot.setParentLoaderPriority(true);
	        webappdocroot.setErrorHandler(new ErrorHandler());
	        webappdocroot.getSessionHandler().getSessionManager().addEventListener(new MySessionListener());
	        contexts.addHandler(webappdocroot);
	        WebAppContext webappdoc = new WebAppContext();
	        webappdoc.setWelcomeFiles(new String[] { 
			        "index.html", "index.jsp"
			    });
			webappdoc.setAttribute("org.eclipse.jetty.webapp.configuration", configuration);
			webappdoc.setResourceBase("docs");
	        webappdoc.setContextPath("/docs");
	        webappdoc.setParentLoaderPriority(true);
	        webappdoc.setErrorHandler(new ErrorHandler());
	        webappdoc.getSessionHandler().getSessionManager().addEventListener(new MySessionListener());
	        contexts.addHandler(webappdoc);

			WebAppContext webrestapi = new WebAppContext();
			webrestapi.setWelcomeFiles(new String[] { 
			        "index.jsp"
			    });
			webrestapi.setAttribute("org.eclipse.jetty.webapp.configuration", configuration);
			webrestapi.setResourceBase("api-rest");
			webrestapi.setContextPath("/api-rest");
			webrestapi.setParentLoaderPriority(true);
			webrestapi.setErrorHandler(new ErrorHandler());
			webrestapi.getSessionHandler().getSessionManager().addEventListener(new MySessionListener());
	        contexts.addHandler(webrestapi);
	        
	        WebSocketServlet wsHandler = new WebSocketServlet()
            {
                /**
				 * 
				 */
				private static final long serialVersionUID = 1L;

				@Override
                public void configure(WebSocketServletFactory factory)
                {
					try {
						factory.getPolicy().setIdleTimeout(Start.SESSION_TIMEOUT);
					} catch ( Exception e) {}
                    factory.register(WebSocketThread.class);
                }
                
            };
            
            ServletContextHandler sockContext = new ServletContextHandler(ServletContextHandler.SESSIONS);
            sockContext.setErrorHandler(new ErrorHandler());
            sockContext.setContextPath("/mentdb/");
            
            ServletHolder holderEvents = new ServletHolder("ws", wsHandler);
            sockContext.addServlet(holderEvents, "/");
	        contexts.addHandler(sockContext);
	        
	        
	        webServer.setHandler(contexts);
	        webServer.start();
			Misc.system_out_print("[OK]", true, "", "");
			Log.write("Starting the web server > [OK]");
			
			Misc.system_out_print("Start the brain > ", false, "", "");
			brain = new Brain();
			if (!(new File("data"+File.separator+".id")).exists()) Misc.create("data"+File.separator+".id", StringFx.generate_random_str("12"));
			brain.start();
			Misc.system_out_print("[OK]", true, "", "");
			Log.write("Start the brain > [OK]");
			
			Misc.system_out_print("Load all languages > ", false, "", "");
			LanguageManager.first("mentdb");
			Misc.system_out_print("[OK]", true, "", "");
			Log.write("Load all languages > [OK]");
			
			Misc.system_out_print("Start all jobs > ", false, "", "");
			JobManager.scheduler_start();
			Misc.system_out_print("[OK]", true, "", "");
			Log.write("Start all jobs > [OK]");
			
			System.out.println("-----------------------------------------------------------------");
			Log.write("-----------------------------------------------------------------");
			Misc.system_out_print("Port "+SERVER_PORT+", Pid "+ManagementFactory.getRuntimeMXBean().getName().split("@", -1)[0]+" > ", false, "", "");
			Misc.system_out_print("[LISTENING]", true, "", "");
			Log.write("Port "+SERVER_PORT+", Pid "+ManagementFactory.getRuntimeMXBean().getName().split("@", -1)[0]+" > [LISTENING]");
			
			//Try to start the server
			try {
				
				boolean executeDocChecker = true;
				
				//Waiting from standard in ...
				while (listening) {

					//Try to start a thread
					try {

						//Accept a connection and start a new thread
						Socket s = serverSocket.accept();
						
						if ((Transaction.logFiles.size()-1)>Start.MAX_SESSION) {
							
							//Increment the number of exceeded sessions
							exceededSessions++;

							//Trace in log file
							Log.write("MentDB: Maximum number of sessions exceeded.");
							
							s.close();
							
						} else {
							
							s.setSoTimeout(SESSION_CONNECT_TIMEOUT);

							//Start the thread
							SessionThread thread = new SessionThread(s);
							thread.start();
	
							//STart the agent
							SessionThreadAgent agent = new SessionThreadAgent(thread);
							agent.start();

						}
						
						//Doc checker
						if (executeDocChecker) {
							
							executeDocChecker = false;
							System.out.println("-----------------------------------------------------------------");
							Log.write("-----------------------------------------------------------------");
							Misc.system_out_print("Documentation > ", false, "", "");
							MQLDocumentation.init();
							MentDBFeature.init();
							Misc.system_out_print("\r"+StringFx.rpad("Documentation >", " ", "61"), false, "", "");
							Misc.system_out_print("[OK]", true, "", "");
							Log.write("Documentation > [OK]");
						
						}

					} catch (Exception e) {
						//Nothing
					}
				}

				System.out.println("-----------------------------------------------------------------");
				Log.write("-----------------------------------------------------------------");
				
				Misc.system_out_print("Stop the brain > ", false, "", "");
				Brain.listening = false;
				//Wait the end of the thread
				while(Start.brain.isAlive()) {
					
					Thread.sleep(200);
					
				}
				Misc.system_out_print("[OK]", true, "", "");
				Log.write("Stop the brain > [OK]");
				
				//Stop the web server
				Misc.system_out_print("Stop the web server > ", false, "", "");
				Start.webServer.setStopTimeout(1);
				Start.webServer.stop();
				Start.webServer.destroy();
				Misc.system_out_print("[OK]", true, "", "");
				Log.write("Stop the web server > [OK]");
				
				Misc.system_out_print("Stop the heartbeat > ", false, "", "");

				Statistic.stopHeartbeat10m();
				Statistic.stopHeartbeat24h();
				
				Misc.system_out_print("[OK]", true, "", "");
				
				Log.write("Stop the heartbeat > [OK]");
				
				Misc.system_out_print("Stop all jobs > ", false, "", "");
				JobManager.scheduler_stop();
				Misc.system_out_print("[OK]", true, "", "");
				Log.write("Stop all jobs > [OK]");
				
				Misc.system_out_print("Close database > ", false, "", "");
				
				//Close the database
				Start.db.close();
				userTransactions = (new File("data"+File.separator+"transaction")).list();
				//Parse all transactions
				for(int itransaction = 0;itransaction<userTransactions.length;itransaction++) {
					try {
						if ((new File("data"+File.separator+"transaction"+File.separator+userTransactions[itransaction]+File.separator+"rollback.log")).exists()) {
							
							Transaction.logFiles.get(userTransactions[itransaction]).close();
							
						}
					} catch (Exception e) {};
				}
				
				Misc.system_out_print("[OK]", true, "", "");
				
				Log.write("Close database > [OK]");

				System.out.println("-----------------------------------------------------------------");
				System.out.println("Bye.");

				Log.write("-----------------------------------------------------------------");
				Log.write("Bye.");

			} catch (IOException e) {

				//An error was generated
				System.err.println("MentDB : Could not listen on port " + Start.SERVER_PORT);

				//Exit
				System.exit(-1);
			}

		} catch(Exception e) {

			e.printStackTrace();

		}

	}

	//The main method to start the database
	public static void main(String args[]) {

		new Start();
		
	}

    private static class MySessionListener implements HttpSessionListener {

		@Override
		public void sessionCreated(HttpSessionEvent se) {
			// TODO Auto-generated method stub
		}

		@Override
		public void sessionDestroyed(HttpSessionEvent se) {
			// TODO Auto-generated method stub
			
			if (((String)se.getSession().getAttribute("target"))!=null) {
				
				if (!((String)se.getSession().getAttribute("target")).equals("mql_editor")) {
					
					UserManager.disconnect((String)se.getSession().getAttribute("username"));
					
				}
			}
			
		}
	}

}
